/*
 * This file is part of SubLink.
 * Copyright (C) 2011  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

var EXPORTED_SYMBOLS = ["AsyncRequest"];
Components.utils.import("resource://sublink-gremio-modules/util.js");

function AsyncRequest() {
  this.requestUrl = null;
  this.onError = null;
  this.onSuccess = null;
  this.onComplete = null;
}

AsyncRequest.prototype._createXhr = function() {
  var xhr = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);

  xhr.open("GET", this.requestUrl, true);
  xhr.setRequestHeader("Accept-Language", null);
  xhr.setRequestHeader("Cookie", null);
  xhr.setRequestHeader("Accept", null);
  xhr.setRequestHeader("User-Agent", 'SubLink/3.0.3 (+http://br.mozdev.org/grem.io/)');
  // LOAD_ONLY_IF_MODIFIED makes xhr results from cache be empty!
  //xhr.channel.loadFlags |= Ci.nsICachingChannel.LOAD_ONLY_IF_MODIFIED;
  return xhr;
};


AsyncRequest.prototype.convert = function(uri) {
  this.requestUrl = "http://grem.io/api/api.php?url=" + encodeURIComponent(uri.spec);

  var xhr = this._createXhr();
  var me = this;
  xhr.onload = function(evt) {
    me.onComplete();
    me.requestSuccessConvert(this); // this == evt.target == evt.currentTarget == xhr;
  }
  xhr.onerror = function(evt) {
    me.onComplete();
    me.requestFailConvert(this);
  }
  xhr.send(null);
};



AsyncRequest.prototype.requestFailConvert = function(xhr) {
  var errorData = {
    id: "noconnection",
    xhr: xhr,
    msg: null
  };
  this.onError(errorData);
};


AsyncRequest.prototype.requestSuccessConvert = function(xhr) {
  var errorData = {
    id: null,
    xhr: xhr,
    msg: null
  };

  var doc = xhr.responseText;
  if (doc === null || doc.length === 0) {
    errorData.id = "invalid";
    this.onError(errorData);
    return;
  }

  if (xhr.status !== 200) {
    errorData.id = "couldntconvert";
    errorData.msg = doc;
    this.onError(errorData);
    return;
  }

  var uri = util.stringToUri(doc);
  if (uri === null) {
    errorData.id = "invalid";
    this.onError(errorData);
    return;
  }

  this.onSuccess({
    shortUri:           uri,
    responseStatusCode: xhr.status
  });
};


AsyncRequest.prototype.findRedir = function(uri) {
  this.requestUrl = uri.spec;
  var xhr = this._createXhr();
  xhr.channel.QueryInterface(Ci.nsIHttpChannel).redirectionLimit = 0;
  var me = this;
  xhr.onload = function(evt) {
    me.onComplete();
    me.requestSuccessRestore(this);
  }
  xhr.onerror = function(evt) {
    me.onComplete();
    me.requestFailRestore(this);
  }
  xhr.send(null);
};


AsyncRequest.prototype.requestFailRestore = function(xhr) {

  if (xhr.status !== 301) { // 0 = no response
    this.onError({
      id: "noconnection",
      xhr: xhr,
      msg: null
    });
    return;
  }


  var uri = util.getLocationHeader(xhr);
  if (uri === null) {
    //util.log("Location: " + url+xhr2);
    this.onError({
      id: "invalid",
      xhr: xhr2, // BUG xhr will be empty
      msg: null
    });
    return;
  }

/*
  var url = xhr.getResponseHeader('Location');
  if (url === null) {
    errorData.id = 'invalid';
    this.onError(errorData);
    return;
  }

  var uri = util.stringToUri(url);
  if (uri === null) {
    errorData.id = 'invalid';
    this.onError(errorData);
    return;
  }
*/
  this.onSuccess({
    longUri:            uri,
    responseStatusCode: xhr.status
  });
};


AsyncRequest.prototype.requestSuccessRestore = function(xhr) {

  if (xhr.status > 400) {
    var errorData = {
      id: 'couldntrestore',
      xhr: xhr,
      msg: xhr.status + ' - ' + xhr.statusText
    };
    this.onError(errorData);

  } else {
    var errorData = {
      id: 'couldntrestore-short',
      xhr: xhr,
      msg: null
    };
    this.onError(errorData);
  }
};
